/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include <cassert>
#include <vector>
#include <shlwapi.h>

#include "executor.h"
#include "Kernel.h"
#include "stlcatalogs.h"
#include "executor_menus.h"
#include "executor_addons.h"
#include "lang.h"
#include "codec_for_values.h"
#include "header.h"
#include "executor_confirmations.h"
#include "strings_utils.h"

extern kr::CKernel *g_pK;
extern struct PluginStartupInfo g_PluginInfo; 
extern struct FarStandardFunctions g_FSF;

namespace {
	inline tstring get_implicit_name()
	{
		tstring name;
		tstring value;
		nf::get_implicit_name_and_value(INVALID_HANDLE_VALUE, true, name, value);
		return name;
	}
	tstring get_current_dir(HANDLE hPlugin, bool bActivePanel = true)
	{
		PanelInfo p;
		g_PluginInfo.Control(hPlugin, bActivePanel
			? FCTL_GETPANELINFO 
			: FCTL_GETANOTHERPANELINFO, &p);

		return p.CurDir;
	}
} //namespace

bool nf::ExecuteCommand(kr::tparsed_command &cmd)
{
	HANDLE hPlugin = INVALID_HANDLE_VALUE;
	kr::tshortcut_info sh;
	kr::tcatalog_info cat;

	switch(cmd.kind) 
	{
	case kr::QK_OPEN_SHORTCUT: //cd:
	//   
		if (! select_required_shortcut(hPlugin, cmd, sh)) return false;
	//   
		return open_shortcut(hPlugin, sh, cmd.local_directory);
		break;
	case kr::QK_OPEN_BY_PATH:	//cd:~ 
		//   
		if (! select_required_shortcut_by_path(hPlugin
			, get_current_dir(hPlugin, true)
			, cmd
			, sh)) return false;
		//   
		return open_shortcut(hPlugin, sh, cmd.local_directory);
		break;
	case kr::QK_OPEN_ENVIRONMENT_VARIABLE: //cd:%
		return open_environment_variable(hPlugin, cmd.shortcut);
	case kr::QK_OPEN_NETWORK:	/*cd:\\*/
		break;
	case kr::QK_INSERT_SHORTCUT:	//cd::
	case kr::QK_INSERT_BOTH:				//cd:::
	case kr::QK_INSERT_SHORTCUT_IMPLICIT: //cd::
	case kr::QK_INSERT_BOTH_TEMPORARY:	//cd:+:
	case kr::QK_INSERT_SHORTCUT_TEMPORARY_IMPLICIT: //cd:+
	case kr::QK_INSERT_SHORTCUT_TEMPORARY: //cd:+
		{
			bool bTemporary = ((kr::QK_INSERT_SHORTCUT_TEMPORARY == cmd.kind) 
							|| (kr::QK_INSERT_BOTH_TEMPORARY == cmd.kind)
							|| (kr::QK_INSERT_SHORTCUT_TEMPORARY_IMPLICIT == cmd.kind));
			bool bImplicit =  ((kr::QK_INSERT_SHORTCUT_TEMPORARY_IMPLICIT == cmd.kind) 
							|| (kr::QK_INSERT_SHORTCUT_IMPLICIT == cmd.kind));

			if (bImplicit) 
				cmd.shortcut = get_implicit_name();

			if (cmd.shortcut.empty())
			{	// 
				g_pK->insert_catalog(cmd.catalog);

			} else {
				//   
				if (! select_required_catalog(hPlugin, cmd, cat)) 
				{
					return false;
				}

				//     
				if ((kr::QK_INSERT_BOTH == cmd.kind) || (kr::QK_INSERT_BOTH_TEMPORARY == cmd.kind))
					return add_shortcut_both(hPlugin, cat, cmd.shortcut, bTemporary, bImplicit);
				else 
					return add_shortcut(hPlugin, cat, cmd.shortcut, bTemporary, bImplicit);
			} //if
		} break;

	case kr::QK_DELETE_SHORTCUT_IMPLICIT:
	case kr::QK_DELETE_SHORTCUT:	//cD:-
		{
			bool bImplicit = kr::QK_DELETE_SHORTCUT_IMPLICIT == cmd.kind;
			
			if (bImplicit)
			{
				//    
				//   ..
				//cmd.shortcut = get_implicit_name();	
				if (! select_required_shortcut_by_path(hPlugin
					, get_current_dir(hPlugin, true)
					, cmd
					, sh)) return false;
			
			} else 
			{
				//   
				if (! select_required_shortcut(hPlugin, cmd, sh)) 
					return false;
			}

			// 
			return delete_shortcut(hPlugin, sh, kr::QK_DELETE_SHORTCUT_IMPLICIT == cmd.kind);
		} break;
	case kr::QK_DELETE_CATALOG:	//cD:-
		//   
		if (! select_required_catalog(hPlugin, cmd, cat)) return false;
// 
		return delete_catalog(hPlugin, cat);
		break;
	default:;
	}
	return false;
}

//      
bool nf::select_required_path(HANDLE hPlugin
							  , nf::tshortcut_value_parsed const &Value
							  , tstring const& PathPattern
							  , tstring &ResultPath)
{
	if (! Value.bValueEnabled) return false;

	kr::tdirs_list list;
	g_pK->select_variants_dirs(Value.value, PathPattern, list);

	if (! list.empty())
		return nf::select_required_path_through_menu(list, ResultPath);

	return false;
}


//      
//    
bool nf::select_required_shortcut(HANDLE hPlugin
								  , kr::tparsed_command const &cmd
								  , kr::tshortcut_info& sh)
{
	while (true)
	{
		kr::tshortcuts_list list;
		kr::tvalues_list values;
		size_t nexact = g_pK->select_variants_shortcuts(cmd.shortcut.c_str(), cmd.catalog.c_str(), list);
		
		bool bExpand = ! list.empty();	//   ..
		
		//     
		//    
		//,    , 
		//    
		//    


		if (nexact == 1)
		{
			bExpand = false;
				if (g_pK->m_S.GetValue(nf::ST_ALWAYS_EXPAND_SHORTCUTS))
					bExpand = true;
		}

		if (bExpand)
		{
			int n = nf::select_required_shortcut_through_menu(list, values, sh);
			if (n == -1)
			{
				//  ..
				delete_shortcut(hPlugin, sh, false);
			} else return n > 0;
		} else {
			if (!list.empty())
				sh = *list.begin();
			return true;
		}
	} //while
}

//      
bool nf::select_required_catalog(HANDLE hPlugin
								 , kr::tparsed_command const &cmd
								 , kr::tcatalog_info &c)
{
	kr::tcatalogs_list list;
	
	if (cmd.catalog.empty()){
		c = _T("");
		return true;		//  -  ...
	}
	
	tstring catalog_name = cmd.catalog;
	remove_trailing_chars(catalog_name, SLASH_CATS_CHAR);
	add_leading_char_if_not_exists(catalog_name, SLASH_CATS_CHAR);
	g_pK->select_variants_catalogs(catalog_name.c_str(), list);

	if (list.empty()) return false;	// 

	//     
	//     -    
	bool bExpand = list.size() > 1;	//   ..
	if (bExpand)
	{
		int n = nf::select_required_catalog_through_menu(list, c);
		return n > 0;
	} else {
		c = *list.begin();
		return true;
	}
}

namespace nf
{
	bool find_best_dir(HANDLE hPlugin
		, nf::tshortcut_value_parsed const &p
		, tstring &dir)
	{	//   
		//    -  
		assert(p.ValueType != nf::VAL_TYPE_PLUGIN_DIRECTORY);	//       

		if (::PathFileExists(p.value.c_str()))
		{
			dir = p.value;
			return true;
		} else {
			//  
			std::vector<TCHAR> buf(p.value.size()+1);
			_tcscpy(&buf[0], p.value.c_str());

			do {
				if (! ::PathRemoveFileSpec(&buf[0])) 
					return false; //   ..
			} while (! ::PathFileExists(&buf[0]));

			//!todo  
			//!    
			if (! nf::confirm_go_to_nearest(hPlugin, p.value.c_str(), &buf[0])) 
				return false;

			dir = &buf[0];
			return true;
		}
	} //find_best_dir

	void close_and_start_another_plugin(HANDLE hPlugin
		, tstring const& Command
		, bool bActivePanel
		, bool bOpenBoth
		)
	{	//""   Command + Enter

	//    
		tstring prefix = g_pK->GetFirstPluginPrefix();

		ULONG add_size = 4 + static_cast<int>(prefix.size()); //TAB + ENTER + TAB + "cd:" + ENTER  
		
		static KeySequence ks;
		ks.Flags = 0;//KSFLAGS_DISABLEOUTPUT;
		ks.Count = static_cast<int>(Command.size()) + 1;
		
		std::vector<DWORD> keys_seq(ks.Count + add_size);
		ks.Sequence = &keys_seq[0];

		if (! bActivePanel)
		{
			UINT j = 0, i;
			keys_seq[j++] = VK_TAB;
			ks.Count += 1;

			for (i = 0; i < Command.size(); ++i, ++j)
				keys_seq[j] = static_cast<DWORD>(Command[i]);
			keys_seq[j++] = VK_RETURN;

			if (! bOpenBoth)
			{
				keys_seq[j++] = VK_TAB;
				for (i = 0; i < prefix.size(); ++i, ++j)
					keys_seq[j] = static_cast<DWORD>(prefix[i]);
				keys_seq[j++] = VK_RETURN;
				ks.Count += 1 + static_cast<int>(prefix.size()) + 1;
			}
		} else 
		{
			for (unsigned int i = 0; i < Command.size(); ++i)
				keys_seq[i] = static_cast<DWORD>(Command[i]);
			keys_seq[i] = VK_RETURN;
		}

		BOOL bSuccess = g_PluginInfo.AdvControl(g_PluginInfo.ModuleNumber
			, ACTL_POSTKEYSEQUENCE
			, reinterpret_cast<void*>(&ks));
		assert(bSuccess);
	}

	bool open_shortcut_on_panel(HANDLE hPlugin
		, nf::tshortcut_value_parsed &panel
		, tstring const &path
		, bool bActivePanel
		, bool bOpenBoth	
		, bool bClosePlugin)
	{
		assert(panel.bValueEnabled);
		//     
		if (! path.empty())
		{
			tstring inap_path;
			nf::select_required_path(hPlugin, panel, path, inap_path);
			panel.value.swap(inap_path);
		}

		if (panel.ValueType == VAL_TYPE_PLUGIN_DIRECTORY)
		{
			//open virtual directory
			close_and_start_another_plugin(hPlugin, panel.value, bActivePanel, bOpenBoth);
		} else {
			//open local directory
			tstring dir;
			switch (panel.ValueType)
			{
			case nf::VAL_ENVIRONMENT_VARIABLE:
				if (! nf::select_path_for_environment_variable(hPlugin, panel.value, dir)) return false;
			break;
			case nf::VAL_DIRECT_PATH:
				dir = panel.value;
			break;
			default:
				if (! nf::find_best_dir(hPlugin, panel, dir)) return false;
			}; //switch
			g_PluginInfo.Control(hPlugin
				, (bActivePanel) 
					? FCTL_SETPANELDIR
					: FCTL_SETANOTHERPANELDIR
				, (void*)dir.c_str());
			g_PluginInfo.Control(hPlugin
				, (bActivePanel) 
					? FCTL_REDRAWPANEL
					: FCTL_REDRAWANOTHERPANEL
				, 0);
			if (bClosePlugin)		
				g_PluginInfo.Control(hPlugin, FCTL_CLOSEPLUGIN, (void*)dir.c_str());
		}; 

		return true;
	}	//open_shortcut_on_panel
} //namespace

//  ,   
bool nf::open_shortcut(HANDLE hPlugin
					   , kr::tshortcut_info const& sh
					   , tstring const &path
					   , bool bOpenOnActivePanel)
{	
	tstring value;
	g_pK->get_value_for_shortcut(sh, value);
	nf::tshortcut_value_parsed ap;		//active panel
	nf::tshortcut_value_parsed inap;	//inactive panel
	if (bOpenOnActivePanel)
	{
		nf::decode_values(value, ap, inap);
	} else	
	{	//  ""
		nf::decode_values(value, inap, ap);
	}

	bool bOpenBoth = ap.bValueEnabled && inap.bValueEnabled;
	if (inap.bValueEnabled)
		open_shortcut_on_panel(hPlugin, inap, path, false, bOpenBoth, false);
	if (ap.bValueEnabled)
		open_shortcut_on_panel(hPlugin, ap, path, true, bOpenBoth, (hPlugin != INVALID_HANDLE_VALUE));

	return true;
}

bool nf::open_specified_path(HANDLE hPlugin, tstring const& path)
{	//     
	nf::tshortcut_value_parsed ap;		//active panel
	ap.bValueEnabled = true;
	ap.value  = path;
	ap.ValueType = nf::VAL_DIRECT_PATH;

	open_shortcut_on_panel(hPlugin, ap, _T(""), true, false, true);
	return true;
}



namespace {
	void open_in_explorer(tstring const& s)
	{
		tstring path = tstring(_T("explorer \"")) + s + _T("\"");
		
		std::vector<TCHAR> sv;
		sv.resize(path.size() + 1);
		::lstrcpy(&sv[0], path.c_str());
		
		STARTUPINFO si;
		memset(&si, 0, sizeof(STARTUPINFO));
		si.cb = sizeof(STARTUPINFO);
		PROCESS_INFORMATION pi;
		::CreateProcess(0, &sv[0], NULL, NULL, 0, 0, NULL, 0, &si, &pi);
	}
}

bool nf::open_shortcut_in_explorer(HANDLE hPlugin
								   , kr::tshortcut_info const& sh
								   , tstring const &path)
{
	tstring value;
	g_pK->get_value_for_shortcut(sh, value);
	nf::tshortcut_value_parsed ap;		//active panel
	nf::tshortcut_value_parsed inap;	//inactive panel
	nf::decode_values(value, ap, inap);

	if (inap.bValueEnabled)
	{
		open_in_explorer(inap.value);
	}

	if (ap.bValueEnabled)
	{
		open_in_explorer(ap.value);
	}

	return TRUE;
}


bool nf::add_shortcut(HANDLE hPlugin
					  , kr::tshortcut_info const &sh
					  , TCHAR const* value
					  , bool bImplicit)
{
	if (sh.shortcut.empty()) return false;	//name of shortcut is absent

	//       
	sc::CCatalog c(sh.catalog);
	tstring v;
	if (c.get_shortcut_info(sh.shortcut, sh.bIsTemporary, v))
	{
		if (! nf::confirm_override(hPlugin, sh, v)) return false;
	}
	if (bImplicit)
		if (! nf::confirm_implicit_insert(hPlugin, sh, value)) return false;

	return g_pK->insert_shortcut(sh, value, true) != 0;
}

bool nf::add_shortcut(HANDLE hPlugin
					  , kr::tcatalog_info const &cat
					  , tstring const& sh
					  , bool bTemporary
					  , bool bImplicit)
{
	if (sh.empty()) return false;	//name of shortcut is absent

	PanelInfo p;
	g_PluginInfo.Control(hPlugin, FCTL_GETPANELINFO, &p);
	
	kr::tshortcut_info info;
	info.catalog = cat;
	info.shortcut = sh;
	info.bIsTemporary = bTemporary;

	return add_shortcut(hPlugin, info, p.CurDir, bImplicit);
}

bool nf::add_shortcut_both(HANDLE hPlugin, kr::tcatalog_info const &cat, tstring const& sh, bool bTemporary, bool bImplicit)
{
	if (sh.empty()) return false;	//name of shortcut is absent

	PanelInfo p1;
	PanelInfo p2;
	g_PluginInfo.Control(hPlugin, FCTL_GETPANELINFO, &p1);
	g_PluginInfo.Control(hPlugin, FCTL_GETANOTHERPANELINFO, &p2);

	tstring value = nf::encode_values(p1.CurDir, p2.CurDir);

	kr::tshortcut_info info;
	info.catalog = cat;
	info.shortcut = sh;
	info.bIsTemporary = bTemporary;

	return add_shortcut(hPlugin, info, value.c_str(), bImplicit);
}


bool nf::delete_shortcut(HANDLE hPlugin, kr::tshortcut_info const& sh, bool bImplicit)
{
	//    
	if (! confirm_delete(hPlugin, sh, bImplicit)) return false;
	return g_pK->delete_shortcut(sh);
}

bool nf::delete_catalog(HANDLE hPlugin, kr::tcatalog_info const &cat)
{	
	sc::CCatalog c(cat);
	bool bIsEmpty = (0 == c.number_subcatalogs()) && (0 == c.number_shortcuts());
	//     
	if (! confirm_delete_catalog(hPlugin, cat, bIsEmpty)) return false;
	return g_pK->delete_catalog(cat);
}


bool nf::add_catalog(HANDLE hPlugin, kr::tcatalog_info const &cat)
{
	tstring parent;
	tstring c;

	kr::tcatalog_info::size_type pos = cat.find_last_of(SLASH_CATS);
	if (pos != tstring::npos)
	{
		parent.assign(cat, 0, pos);
		c.assign(cat, pos+1, cat.size()-pos);
	} else 
		c = cat;
	return g_pK->insert_catalog(c.c_str(), parent.c_str());
}


bool nf::get_implicit_name_and_value(HANDLE hPlugin
									 , bool bGetDataFromActivePanel
									 , tstring &name
									 , tstring &value)
{
	value = get_current_dir(hPlugin, bGetDataFromActivePanel);
	tstring s = ::PathFindFileName(value.c_str());

	tstring::iterator ps = s.begin();
	while (ps != s.end())
	{ //first char shouldn't be command characther - skip if it is one.
		switch(*ps) {
		case _T('%'):
//		case _T('&'):
		case _T('+'):
		case _T('|'):
		case _T('-'):
		case _T('!'):
			s.erase(ps);
			ps = s.begin();
			continue;
		};
		break;
	}

	//other characters should't be & (FAR) and %, | (NF) 
	while (ps != s.end())
	{
		switch(*ps) {
		case _T('%'):
//		case _T('&'):
		case _T('|'):
			*ps = _T('_');
		};
		++ps;
	}

	name = s;
	return true;
}
