#include "StdAfx.h"
#include "nf.h"
#include "Registry.h"
#include <cassert>
#include "Kernel.h"

using namespace nf;

namespace 
{
	typedef struct tdefault_flag_value
	{
		tsetting_flags flag;
		bool value;
		TCHAR const *regkey;
	} _default_flag_value;

	tdefault_flag_value default_flags_values [] = 
	{
		{FG_SHOW_ERRORS, 0, _T("ShowErrors")}
		,{FG_SHOW_IN_PLUGINS_MENU, 1, _T("ShowInPluginsMenu")}
		,{FG_SHOW_IN_DISK_MENU, 1, _T("ShowInDiskMenu")}
		,{FG_SHOW_TEMPORARY_AS_HIDDEN, 1, _T("ShowTemporaryNamesAsHideFiles")}
		,{FG_CONFIRM_DELETE, 1, _T("CONFIRM_DELETE")}
		,{FG_CONFIRM_OVERRIDE, 1, _T("CONFIRM_OVERRIDE")}
		,{FG_CONFIRM_IMPLICIT_CREATION, 1, _T("CONFIRM_IMPLICIT_CREATION")}
		,{FG_PANEL_SHOW_EXTENSIONS, 1, _T("PANEL_SHOW_EXTENSIONS")}
	};
	const int NUMBER_FLAGS = sizeof(default_flags_values) / sizeof(tdefault_flag_value);


	TCHAR const * DEFAULT_PREFIX = "cc:";
}

CSettings::CSettings()
{
	m_FV.resize(NUMBER_FLAGS);
	assert(g_PluginInfo.RootKey);
	m_nf_reg_key = tstring(g_PluginInfo.RootKey) + _T("\\NamedFolders");
	reload();
	m_plugin_prefixes = _T("cc:ccc:");
}

CSettings::~CSettings(void)
{
	
}


void CSettings::reload()
{	//   
	CRegistry reg(HKEY_CURRENT_USER, m_nf_reg_key.c_str());
	for (int i = 0; i < NUMBER_FLAGS; ++i)
	{
		tdefault_flag_value const &d = default_flags_values[i];
	//   
		DWORD value;
		if (reg.GetValue(d.regkey, value))
			m_FV[d.flag] = value;
		else 
			m_FV[d.flag] = d.value;	//  -
	}
}

void CSettings::save()
{
	CRegistry reg(HKEY_CURRENT_USER, m_nf_reg_key.c_str());
	for (int i = 0; i < NUMBER_FLAGS; ++i)
	{
		tdefault_flag_value const &d = default_flags_values[i];
		reg.SetValue(d.regkey, m_FV[d.flag]);
	}
}

bool CSettings::get_set_value(tsetting_flags value) const
{	//    
	assert(static_cast<UINT>(value) < m_FV.size());
	return m_FV[static_cast<UINT>(value)];
}

char const* CSettings::get_CommandPrefixes() const
{	//    PluginInfo
	return m_plugin_prefixes.c_str();
}
