/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "settings.h"
#include "Registry.h"
#include <cassert>
#include "Kernel.h"

using namespace nf;

namespace 
{
	typedef struct tdefault_string_value
	{
		tsetting_strings flag;
		TCHAR const* default_value;
		TCHAR const *regkey;
	} _tdefault_string_value;

	tdefault_string_value const default_strings_values [] = 
	{
		{STS_PREFIXES, _T("cd:cc:"), _T("prefix")}
	};

	typedef struct tdefault_flag_value
	{
		tsetting_flags flag;
		DWORD default_value;
		TCHAR const *regkey;
	} _default_flag_value;

	tdefault_flag_value const default_flags_values [] = 
	{
		{ST_SHOW_ERRORS, 0, _T("ShowErrors")}
		,{ST_SHOW_IN_PLUGINS_MENU, 1, _T("ShowInPluginsMenu")}
		,{ST_SHOW_IN_DISK_MENU, 1, _T("ShowInDiskMenu")}
		,{ST_SHOW_TEMPORARY_AS_HIDDEN, 1, _T("ShowTemporaryNamesAsHideFiles")}
		,{ST_CONFIRM_DELETE, 1, _T("CONFIRM_DELETE")}
		,{ST_CONFIRM_DELETE_CATALOGS, 1, _T("CONFIRM_DELETE_CATALOGS")}
		,{ST_CONFIRM_DELETE_NOT_EMPTY_CATALOGS, 1, _T("CONFIRM_DELETE_NOT_EMPTY_CATALOGS")}
		,{ST_CONFIRM_OVERRIDE, 1, _T("CONFIRM_OVERRIDE")}
		,{ST_CONFIRM_IMPLICIT_CREATION, 1, _T("CONFIRM_IMPLICIT_CREATION")}
		,{ST_CONFIRM_IMPLICIT_DELETION, 1, _T("CONFIRM_IMPLICIT_DELETION")}
		,{ST_CONFIRM_GO_TO_NEAREST, 1, _T("CONFIRM_GO_TO_NEAREST")}
		,{ST_SHOW_KEYS_IN_MENU, 1, 	_T("CONFIRM_KEYS_IN_MENU")}
		,{ST_SELECT_SH_MENU_MODE, 1, _T("SH_MENU_MODE")}
		,{ST_PANEL_MODE, 6, _T("PANEL_MODE")}
		,{ST_SELECT_CATS_MENU_MODE, 1, _T("CATS_MENU_MODE")}
		,{ST_ALWAYS_EXPAND_SHORTCUTS, 1, _T("ALWAYS_EXPAND_SHORTCUTS")}
		,{ST_HISTORY_IN_DIALOG_APPLY_COMMAND, 1, _T("USE_HISTORY_IN_DIALOG_APPLY_COMMAND")}
		,{ST_SELECT_SH_MENU_MODE_EV, 1, _T("SH_MENU_MODE_EV")}
	};

	const int NUMBER_FLAGS = 
		sizeof(default_flags_values) / sizeof(tdefault_flag_value);
	
	const int NUMBER_STRINGS = 
		sizeof(default_strings_values) / sizeof(tdefault_string_value);

}

CSettings::CSettings()
{
	assert(NUMBER_STRINGS == NUMBER_STRING_SETTINGS);
	assert(NUMBER_FLAGS == NUMBER_FLAG_SETTINGS);

	m_FV.resize(NUMBER_FLAGS);
	m_SV.resize(NUMBER_STRINGS);
	
	assert(g_PluginInfo.RootKey);
	m_nf_reg_key = tstring(g_PluginInfo.RootKey);
	m_nf_reg_key += _T("\\NamedFolders");
	reload();
}

CSettings::~CSettings(void)
{
	
}


void CSettings::reload()
{	//   
	CRegistry reg(HKEY_CURRENT_USER, m_nf_reg_key.c_str());
	for (int i = 0; i < NUMBER_FLAGS; ++i)
	{
		tdefault_flag_value const &d = default_flags_values[i];
	//   
		DWORD value;
		if (reg.GetValue(d.regkey, value)) m_FV[d.flag] = value;
		else m_FV[d.flag] = d.default_value;	//  -
	}
	for (int i = 0; i < NUMBER_STRINGS; ++i)
	{
		tdefault_string_value const &d = default_strings_values[i];
		tstring value;
		if (reg.GetValue(d.regkey, value)) 
			m_SV[d.flag] = value;
		else 
			m_SV[d.flag] = d.default_value;	
	}

}

void CSettings::save()
{
	CRegistry reg(HKEY_CURRENT_USER, m_nf_reg_key.c_str());
	for (int i = 0; i < NUMBER_FLAGS; ++i)
	{
		tdefault_flag_value const &d = default_flags_values[i];
		reg.SetValue(d.regkey, m_FV[d.flag]);
	}
	for (int i = 0; i < NUMBER_STRINGS; ++i)
	{
		tdefault_string_value const &d = default_strings_values[i];
		reg.SetValue(d.regkey, m_SV[d.flag].c_str());
	}
}

DWORD CSettings::GetValue(tsetting_flags fg) const
{	//    
	assert(static_cast<UINT>(fg) < m_FV.size());
	return m_FV[static_cast<DWORD>(fg)];
}

void CSettings::SetValue(tsetting_flags fg, DWORD Value) 
{	//     (  )
	assert(static_cast<UINT>(fg) < m_FV.size());
	m_FV[static_cast<UINT>(fg)] = Value;
}

tstring const& CSettings::GetValue(tsetting_strings fg) const
{	//    
	assert(static_cast<UINT>(fg) < m_SV.size());
	return m_SV[static_cast<DWORD>(fg)];
}

void CSettings::SetValue(tsetting_strings fg, tstring Value) 
{	//     (  )
	assert(static_cast<UINT>(fg) < m_SV.size());
	m_SV[static_cast<UINT>(fg)] = Value;
}

